@@ -24,11 +24,13 @@ from calibre.utils.icu import numeric_sort_key
 COLLAPSE = re.compile(r'[ \t\r\n\v]+')
 STRIPNUM = re.compile(r'[-0-9]+$')
 
+
 def asfloat(value, default):
     if not isinstance(value, (int, long, float)):
         value = default
     return float(value)
 
+
 def dynamic_rescale_factor(node):
     classes = node.get('class', '').split(' ')
     classes = [x.replace('calibre_rescale_', '') for x in classes if
@@ -45,6 +47,7 @@ def dynamic_rescale_factor(node):
 
 
 class KeyMapper(object):
+
     def __init__(self, sbase, dbase, dkey):
         self.sbase = float(sbase)
         self.dprop = [(self.relate(x, dbase), float(x)) for x in dkey]
@@ -92,7 +95,9 @@ class KeyMapper(object):
         dsize = min(diff)[1]
         return dsize
 
+
 class ScaleMapper(object):
+
     def __init__(self, sbase, dbase):
         self.dscale = float(dbase) / float(sbase)
 
@@ -101,13 +106,16 @@ class ScaleMapper(object):
         dsize = ssize * self.dscale
         return dsize
 
+
 class NullMapper(object):
+
     def __init__(self):
         pass
 
     def __getitem__(self, ssize):
         return ssize
 
+
 def FontMapper(sbase=None, dbase=None, dkey=None):
     if sbase and dbase and dkey:
         return KeyMapper(sbase, dbase, dkey)
@@ -116,6 +124,7 @@ def FontMapper(sbase=None, dbase=None, dkey=None):
     else:
         return NullMapper()
 
+
 class EmbedFontsCSSRules(object):
 
     def __init__(self, body_font_family, rules):
@@ -134,7 +143,9 @@ class EmbedFontsCSSRules(object):
                     data=sheet).href
         return self.href
 
+
 class CSSFlattener(object):
+
     def __init__(self, fbase=None, fkey=None, lineh=None, unfloat=False,
                  untable=False, page_break_on_body=False, specializer=None,
                  transform_css_rules=()):
@@ -160,8 +171,14 @@ class CSSFlattener(object):
 
     def __call__(self, oeb, context):
         oeb.logger.info('Flattening CSS and remapping font sizes...')
-        self.context = self.opts =context
+        self.context = self.opts = context
         self.oeb = oeb
+        self.items = list(self.oeb.spine)
+        titlepage = self.oeb.guide.get('titlepage')
+        if titlepage is not None:
+            titlepage = titlepage.item
+            if titlepage is not None and titlepage not in self.items:
+                self.items.append(titlepage)
 
         self.filter_css = frozenset()
         if self.opts.filter_css:
@@ -249,7 +266,7 @@ class CSSFlattener(object):
         self.stylizers = {}
         profile = self.context.source
         css = ''
-        for item in self.oeb.spine:
+        for item in self.items:
             html = item.data
             body = html.find(XHTML('body'))
             if 'style' in html.attrib:
@@ -289,7 +306,7 @@ class CSSFlattener(object):
 
     def baseline_spine(self):
         sizes = defaultdict(float)
-        for item in self.oeb.spine:
+        for item in self.items:
             html = item.data
             stylizer = self.stylizers[item]
             body = html.find(XHTML('body'))
@@ -344,6 +361,20 @@ class CSSFlattener(object):
         if 'align' in node.attrib:
             if tag != 'img':
                 cssdict['text-align'] = node.attrib['align']
+                if cssdict['text-align'] == 'center':
+                    # align=center causes tables to be center aligned,
+                    # which text-align does not. And the ever trustworthy Word
+                    # uses this construct in its HTML output. See
+                    # https://bugs.launchpad.net/bugs/1569583
+                    if tag == 'table':
+                        if 'margin-left' not in cssdict and 'margin-right' not in cssdict:
+                            cssdict['margin-left'] = cssdict['margin-right'] = 'auto'
+                    else:
+                        for table in node.iterchildren(XHTML("table")):
+                            ts = stylizer.style(table)
+                            if ts.get('margin-left') is None and ts.get('margin-right') is None:
+                                ts.set('margin-left', 'auto')
+                                ts.set('margin-right', 'auto')
             else:
                 val = node.attrib['align']
                 if val in ('middle', 'bottom', 'top'):
@@ -351,6 +382,10 @@ class CSSFlattener(object):
                 elif val in ('left', 'right'):
                     cssdict['float'] = val
             del node.attrib['align']
+        if 'valign' in node.attrib and tag == 'td':
+            if cssdict.get('vertical-align') == 'inherit':
+                cssdict['vertical-align'] = node.attrib['valign']
+            del node.attrib['valign']
         if node.tag == XHTML('font'):
             tags = ['descendant::h:%s'%x for x in ('p', 'div', 'table', 'h1',
                 'h2', 'h3', 'h4', 'h5', 'h6', 'ol', 'ul', 'dl', 'blockquote')]
@@ -401,7 +436,7 @@ class CSSFlattener(object):
         fsize = font_size
         is_drop_cap = (cssdict.get('float', None) == 'left' and 'font-size' in
                        cssdict and len(node) == 0 and node.text and
-                       len(node.text) == 1)
+                       (len(node.text) == 1 or (len(node.text) == 2 and 0x2000 <= ord(node.text[0]) <= 0x206f)))
         # Detect drop caps generated by the docx input plugin
         if (node.tag and node.tag.endswith('}p') and len(node) == 0 and node.text and len(node.text.strip()) == 1 and
             not node.tail and 'line-height' in cssdict and 'font-size' in cssdict):
@@ -528,6 +563,7 @@ class CSSFlattener(object):
     def flatten_head(self, item, href, global_href):
         html = item.data
         head = html.find(XHTML('head'))
+
         def safe_lower(x):
             try:
                 x = x.lower()
@@ -569,7 +605,7 @@ class CSSFlattener(object):
 
     def collect_global_css(self):
         global_css = defaultdict(list)
-        for item in self.oeb.spine:
+        for item in self.items:
             stylizer = self.stylizers[item]
             if float(self.context.margin_top) >= 0:
                 stylizer.page_rule['margin-top'] = '%gpt'%\
@@ -608,7 +644,7 @@ class CSSFlattener(object):
     def flatten_spine(self):
         names = defaultdict(int)
         styles, pseudo_styles = {}, defaultdict(dict)
-        for item in self.oeb.spine:
+        for item in self.items:
             html = item.data
             stylizer = self.stylizers[item]
             if self.specializer is not None:
@@ -631,7 +667,6 @@ class CSSFlattener(object):
 
         href = self.replace_css(css)
         global_css = self.collect_global_css()
-        for item in self.oeb.spine:
+        for item in self.items:
             stylizer = self.stylizers[item]
             self.flatten_head(item, href, global_css[item])
-
